@@ -95,7 +95,8 @@ static EelEnumerationEntry file_size_enum_entries[] = {
 	{ "3145728",	    N_("3 MB"),		3145728 },
 	{ "5242880",	    N_("5 MB"),		5242880 },
 	{ "10485760",	    N_("10 MB"),	10485760 },
-	{ "104857600",	    N_("100 MB"),	104857600 }
+	{ "104857600",	    N_("100 MB"),	104857600 },
+	{ "1073741824",     N_("1 GB"),         1073741824 }
 };
 
 static EelEnumerationEntry click_policy_enum_entries[] = {
@@ -170,6 +171,11 @@ static EelEnumerationEntry date_format_entries[] = {
 	{ "informal",	   "Informal",		NAUTILUS_DATE_FORMAT_INFORMAL }
 };
 
+static EelEnumerationEntry new_tab_position_entries[] = {
+	{ "after_current_tab",  "After Current Tab",	NAUTILUS_NEW_TAB_POSITION_AFTER_CURRENT_TAB },
+	{ "end",		"End",			NAUTILUS_NEW_TAB_POSITION_END }
+};
+
 /*
  * A callback which can be used to fetch dynamic fallback values.
  * For example, values that are dependent on the environment (such as user name) 
@@ -285,7 +291,7 @@ static const PreferenceDefault preference_defaults[] = {
 	},
 	{ NAUTILUS_PREFERENCES_IMAGE_FILE_THUMBNAIL_LIMIT,
 	  PREFERENCE_INTEGER,
-	  GINT_TO_POINTER(3145728),
+	  GINT_TO_POINTER(10485760),
 	  NULL, NULL,
 	  "file_size"
 	},
@@ -332,6 +338,16 @@ static const PreferenceDefault preference_defaults[] = {
 	  PREFERENCE_BOOLEAN,
 	  GINT_TO_POINTER (FALSE)
 	},
+	{ NAUTILUS_PREFERENCES_ENABLE_TABS,
+	  PREFERENCE_BOOLEAN,
+	  GINT_TO_POINTER (TRUE)
+	},
+	{ NAUTILUS_PREFERENCES_NEW_TAB_POSITION,
+	  PREFERENCE_STRING,
+	  "after_current_tab",
+	  NULL, NULL,
+	  "new_tab_position"
+	},
 	{ NAUTILUS_PREFERENCES_START_WITH_TOOLBAR,
 	  PREFERENCE_BOOLEAN,
 	  GINT_TO_POINTER (TRUE)
@@ -603,6 +619,9 @@ global_preferences_register_enumerations (void)
 	eel_enumeration_register ("date_format",
 				  date_format_entries,
 				  G_N_ELEMENTS (date_format_entries));
+	eel_enumeration_register ("new_tab_position",
+				  new_tab_position_entries,
+				  G_N_ELEMENTS (new_tab_position_entries));
 
 	/* Set the enumeration ids for preferences that need them */
 	for (i = 0; preference_defaults[i].name != NULL; i++) {
