@@ -59,7 +59,4 @@ void   nautilus_icon_dnd_begin_drag            (NautilusIconContainer *container
 						int                    start_y);
 void   nautilus_icon_dnd_end_drag              (NautilusIconContainer *container);
 
-GList *nautilus_icon_dnd_uri_list_extract_uris (const char            *uri_list);
-void   nautilus_icon_dnd_uri_list_free_strings (GList                 *list);
-
 #endif /* NAUTILUS_ICON_DND_H */
