@@ -32,7 +32,13 @@
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-names.h>
 #include <libnautilus-private/nautilus-trash-monitor.h>
+#include <libnautilus-private/nautilus-marshal.h>
+#include <libnautilus-private/nautilus-dnd.h>
+#include <libnautilus-private/nautilus-icon-dnd.h>
 #include "nautilus-pathbar.h"
+#include "nautilus-window.h"
+#include "nautilus-window-private.h"
+#include "nautilus-window-slot.h"
 
 enum {
         PATH_CLICKED,
@@ -67,6 +73,8 @@ struct _ButtonData
         ButtonType type;
         char *dir_name;
         GFile *path;
+	NautilusFile *file;
+	unsigned int file_changed_signal_id;
 
 	/* custom icon */ 
 	GdkPixbuf *custom_icon;
@@ -78,6 +86,8 @@ struct _ButtonData
         GtkWidget *label;
         guint ignore_changes : 1;
         guint file_is_hidden : 1;
+
+	NautilusDragSlotProxyInfo drag_info;
 };
 
 /* This macro is used to check if a button can be used as a fake root.
@@ -125,8 +135,8 @@ static void     nautilus_path_bar_style_set                (GtkWidget       *wid
 static void     nautilus_path_bar_screen_changed           (GtkWidget       *widget,
 							    GdkScreen       *previous_screen);
 static void     nautilus_path_bar_check_icon_theme         (NautilusPathBar *path_bar);
-static void     nautilus_path_bar_update_button_appearance (NautilusPathBar *path_bar,
-							    ButtonData      *button_data,
+static void     nautilus_path_bar_update_button_appearance (ButtonData      *button_data);
+static void     nautilus_path_bar_update_button_state      (ButtonData      *button_data,
 							    gboolean         current_dir);
 static gboolean nautilus_path_bar_update_path              (NautilusPathBar *path_bar,
 							    GFile           *file_path);
@@ -182,11 +192,6 @@ desktop_location_changed_callback (gpointer user_data)
 	path_bar->desktop_path = nautilus_get_desktop_location ();
 	path_bar->home_path = g_file_new_for_path (g_get_home_dir ());
 	desktop_is_home = g_file_equal (path_bar->home_path, path_bar->desktop_path);
-
-        if (path_bar->home_icon) {
-                g_object_unref (path_bar->home_icon);
-                path_bar->home_icon = NULL;
-        }
 	
 	update_button_types (path_bar);
 }
@@ -217,6 +222,70 @@ trash_state_changed_cb (NautilusTrashMonitor *monitor,
         g_object_unref (file);
 }
 
+static gboolean
+slider_timeout (gpointer user_data)
+{
+	NautilusPathBar *path_bar;
+
+	path_bar = NAUTILUS_PATH_BAR (user_data);
+
+	path_bar->drag_slider_timeout = 0;
+
+	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (path_bar))) {
+		if (path_bar->drag_slider_timeout_for_up_button) {
+			nautilus_path_bar_scroll_up (path_bar->up_slider_button, path_bar);
+		} else {
+			nautilus_path_bar_scroll_down (path_bar->down_slider_button, path_bar);
+		}
+	}
+
+	return FALSE;
+}
+
+static void
+nautilus_path_bar_slider_drag_motion (GtkWidget      *widget,
+				      GdkDragContext *context,
+				      int             x,
+				      int             y,
+				      unsigned int    time,
+				      gpointer        user_data)
+{
+	NautilusPathBar *path_bar;
+	GtkSettings *settings;
+	unsigned int timeout;
+
+	path_bar = NAUTILUS_PATH_BAR (user_data);
+
+	if (path_bar->drag_slider_timeout == 0) {
+		settings = gtk_widget_get_settings (widget);
+
+		g_object_get (settings, "gtk-timeout-expand", &timeout, NULL);
+		path_bar->drag_slider_timeout =
+			g_timeout_add (timeout,
+				       slider_timeout,
+				       path_bar);
+
+		path_bar->drag_slider_timeout_for_up_button =
+			widget == path_bar->up_slider_button;
+	}
+}
+
+static void
+nautilus_path_bar_slider_drag_leave (GtkWidget      *widget,
+				     GdkDragContext *context,
+				     unsigned int    time,
+				     gpointer        user_data)
+{
+	NautilusPathBar *path_bar;
+
+	path_bar = NAUTILUS_PATH_BAR (user_data);
+
+	if (path_bar->drag_slider_timeout != 0) {
+		g_source_remove (path_bar->drag_slider_timeout);
+		path_bar->drag_slider_timeout = 0;
+	}
+}
+
 static void
 nautilus_path_bar_init (NautilusPathBar *path_bar)
 {
@@ -250,6 +319,30 @@ nautilus_path_bar_init (NautilusPathBar *path_bar)
         g_signal_connect (path_bar->down_slider_button, "button_press_event", G_CALLBACK (nautilus_path_bar_slider_button_press), path_bar);
         g_signal_connect (path_bar->down_slider_button, "button_release_event", G_CALLBACK (nautilus_path_bar_slider_button_release), path_bar);
 
+	gtk_drag_dest_set (GTK_WIDGET (path_bar->up_slider_button),
+			   0, NULL, 0, 0);
+	gtk_drag_dest_set_track_motion (GTK_WIDGET (path_bar->up_slider_button), TRUE);
+	g_signal_connect (path_bar->up_slider_button,
+			  "drag-motion",
+			  G_CALLBACK (nautilus_path_bar_slider_drag_motion),
+			  path_bar);
+	g_signal_connect (path_bar->up_slider_button,
+			  "drag-leave",
+			  G_CALLBACK (nautilus_path_bar_slider_drag_leave),
+			  path_bar);
+
+	gtk_drag_dest_set (GTK_WIDGET (path_bar->down_slider_button),
+			   0, NULL, 0, 0);
+	gtk_drag_dest_set_track_motion (GTK_WIDGET (path_bar->up_slider_button), TRUE);
+	g_signal_connect (path_bar->down_slider_button,
+			  "drag-motion",
+			  G_CALLBACK (nautilus_path_bar_slider_drag_motion),
+			  path_bar);
+	g_signal_connect (path_bar->down_slider_button,
+			  "drag-leave",
+			  G_CALLBACK (nautilus_path_bar_slider_drag_leave),
+			  path_bar);
+
         g_signal_connect (nautilus_trash_monitor_get (),
                           "trash_state_changed",
                           G_CALLBACK (trash_state_changed_cb),
@@ -314,6 +407,11 @@ nautilus_path_bar_finalize (GObject *object)
 
 	nautilus_path_bar_stop_scrolling (path_bar);
 
+	if (path_bar->drag_slider_timeout != 0) {
+		g_source_remove (path_bar->drag_slider_timeout);
+		path_bar->drag_slider_timeout = 0;
+	}
+
         g_list_free (path_bar->button_list);
 	if (path_bar->root_path) {
 		g_object_unref (path_bar->root_path);
@@ -328,19 +426,6 @@ nautilus_path_bar_finalize (GObject *object)
 		path_bar->desktop_path = NULL;
 	}
 
-	if (path_bar->root_icon) {
-		g_object_unref (path_bar->root_icon);
-		path_bar->root_icon = NULL;
-	}
-	if (path_bar->home_icon) {
-                g_object_unref (path_bar->home_icon);
-		path_bar->home_icon = NULL;
-	}
-	if (path_bar->desktop_icon) {
-		g_object_unref (path_bar->desktop_icon);
-		path_bar->desktop_icon = NULL;
-	}
-
 	g_signal_handlers_disconnect_by_func (nautilus_trash_monitor_get (),
 					      trash_state_changed_cb, path_bar);
 
@@ -942,28 +1027,12 @@ reload_icons (NautilusPathBar *path_bar)
 {
         GList *list;
 
-        if (path_bar->root_icon) {
-                g_object_unref (path_bar->root_icon);
-                path_bar->root_icon = NULL;
-        }
-        if (path_bar->home_icon) {
-                g_object_unref (path_bar->home_icon);
-                path_bar->home_icon = NULL;
-        }
-        if (path_bar->desktop_icon) {
-                g_object_unref (path_bar->desktop_icon);
-                path_bar->desktop_icon = NULL;
-        }
-
-
         for (list = path_bar->button_list; list; list = list->next) {
                 ButtonData *button_data;
-                gboolean current_dir;
 
                 button_data = BUTTON_DATA (list->data);
-		if (button_data->type != NORMAL_BUTTON) {
-                	current_dir = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_data->button));
-                	nautilus_path_bar_update_button_appearance (path_bar, button_data, current_dir);
+		if (button_data->type != NORMAL_BUTTON || button_data->is_base_dir) {
+                	nautilus_path_bar_update_button_appearance (button_data);
 		}
 
         }
@@ -1049,64 +1118,69 @@ button_clicked_cb (GtkWidget *button,
         g_signal_emit (path_bar, path_bar_signals [PATH_CLICKED], 0, button_data->path);
 }
 
-static GdkPixbuf *
-get_icon_for_file_path (GFile *location, const char *NAUTILUS_ICON_FOLDER_name)
+static NautilusIconInfo *
+get_custom_user_icon_info (ButtonData *button_data)
 {
-	NautilusFile *file;
-	NautilusIconInfo *info;
-	GdkPixbuf *pixbuf;
+/* Bug 80925: With tiny display sizes we get huge memory allocations. */
+#if 0
+	NautilusIconInfo *icon_info;
+	GFile *icon_file;
+	GIcon *icon;
+	char *custom_icon_uri;
 
-	file = nautilus_file_get (location);
-	
-	if (file != NULL &&
-	    nautilus_file_check_if_ready (file,
-					  NAUTILUS_FILE_ATTRIBUTES_FOR_ICON)) {
-		info = nautilus_file_get_icon (file, NAUTILUS_PATH_BAR_ICON_SIZE, 0);
-		pixbuf = nautilus_icon_info_get_pixbuf_at_size (info, NAUTILUS_PATH_BAR_ICON_SIZE);
-		g_object_unref (info);
-		return pixbuf;
+	icon = NULL;
+
+	if (button_data->file != NULL) {
+		custom_icon_uri = nautilus_file_get_custom_icon (button_data->file);
+		if (custom_icon_uri != NULL) {
+			icon_file = g_file_new_for_uri (custom_icon_uri);
+
+			if (g_file_is_native (icon_file)) {
+				icon = g_file_icon_new (icon_file);
+			}
+
+			g_object_unref (icon_file);
+			g_free (custom_icon_uri);
+		}
 	}
 
-	nautilus_file_unref (file);
+	if (icon != NULL) {
+		icon_info = nautilus_icon_info_lookup (icon, NAUTILUS_PATH_BAR_ICON_SIZE);
+		g_object_unref (icon);
 
-	info = nautilus_icon_info_lookup_from_name (NAUTILUS_ICON_FOLDER_name, NAUTILUS_PATH_BAR_ICON_SIZE);
-	pixbuf = nautilus_icon_info_get_pixbuf_at_size (info, NAUTILUS_PATH_BAR_ICON_SIZE);
-	g_object_unref (info);
-	
-	return pixbuf;
+		return icon_info;
+	}
+#endif
+
+	return NULL;
 }
 
-static GdkPixbuf *
-get_button_image (NautilusPathBar *path_bar,
-		  ButtonType  button_type)
+static NautilusIconInfo *
+get_type_icon_info (ButtonData *button_data)
 {
-	switch (button_type)
+	switch (button_data->type)
         {
 		case ROOT_BUTTON:
-			if (path_bar->root_icon != NULL) {
-				return path_bar->root_icon;
-                       	}
-
-			path_bar->root_icon = get_icon_for_file_path (path_bar->root_path, NAUTILUS_ICON_FILESYSTEM);
-			return path_bar->root_icon;
+			return nautilus_icon_info_lookup_from_name (NAUTILUS_ICON_FILESYSTEM,
+								    NAUTILUS_PATH_BAR_ICON_SIZE);
 
 		case HOME_BUTTON:
-		      	if (path_bar->home_icon != NULL) {
-		      		return path_bar->home_icon;
-			}
-
-			path_bar->home_icon = get_icon_for_file_path (path_bar->root_path, NAUTILUS_ICON_HOME);
-			return path_bar->home_icon;
+			return nautilus_icon_info_lookup_from_name (NAUTILUS_ICON_HOME,
+								    NAUTILUS_PATH_BAR_ICON_SIZE);
 
                 case DESKTOP_BUTTON:
-                      	if (path_bar->desktop_icon != NULL) {
-				return path_bar->desktop_icon;
+			return nautilus_icon_info_lookup_from_name (NAUTILUS_ICON_DESKTOP,
+								    NAUTILUS_PATH_BAR_ICON_SIZE);
+
+                case NORMAL_BUTTON:
+			if (button_data->is_base_dir) {
+				return nautilus_file_get_icon (button_data->file,
+							       NAUTILUS_PATH_BAR_ICON_SIZE,
+							       NAUTILUS_FILE_ICON_FLAGS_NONE);
 			}
-			path_bar->desktop_icon = get_icon_for_file_path (path_bar->root_path, NAUTILUS_ICON_DESKTOP);
-      			return path_bar->desktop_icon;
 
 	    	default:
-                       return NULL;
+			return NULL;
         }
   
        	return NULL;
@@ -1120,6 +1194,16 @@ button_data_free (ButtonData *button_data)
 	if (button_data->custom_icon) {
 		g_object_unref (button_data->custom_icon);
 	}
+	if (button_data->file != NULL) {
+		g_signal_handler_disconnect (button_data->file,
+					     button_data->file_changed_signal_id);
+		nautilus_file_monitor_remove (button_data->file, button_data);
+		nautilus_file_unref (button_data->file);
+	}
+
+	g_object_unref (button_data->drag_info.target_location);
+	button_data->drag_info.target_location = NULL;
+
         g_free (button_data);
 }
 
@@ -1161,15 +1245,14 @@ label_size_request_cb (GtkWidget       *widget,
 }
 
 static void
-nautilus_path_bar_update_button_appearance (NautilusPathBar *path_bar,
-				            ButtonData *button_data,
-				            gboolean    current_dir)
+nautilus_path_bar_update_button_appearance (ButtonData *button_data)
 {
+	NautilusIconInfo *icon_info;
+	GdkPixbuf *pixbuf;
         const gchar *dir_name = get_dir_name (button_data);
-	
 
         if (button_data->label != NULL) {
-                if (current_dir) {
+                if (gtk_label_get_use_markup (GTK_LABEL (button_data->label))) {
 			char *markup;
 
 	  		markup = g_markup_printf_escaped ("<b>%s</b>", dir_name);
@@ -1181,19 +1264,44 @@ nautilus_path_bar_update_button_appearance (NautilusPathBar *path_bar,
         }
 
         if (button_data->image != NULL) {
-		if (button_data->type == MOUNT_BUTTON || (button_data->type == NORMAL_BUTTON && button_data->is_base_dir) ) {
-	
-			/* set custom icon for roots */
-			if (button_data->custom_icon) {
-				gtk_image_set_from_pixbuf (GTK_IMAGE (button_data->image), button_data->custom_icon);  
-			}
+		if (button_data->custom_icon) {
+			gtk_image_set_from_pixbuf (GTK_IMAGE (button_data->image), button_data->custom_icon);  
+			gtk_widget_show (GTK_WIDGET (button_data->image));
 		} else {
-	                GdkPixbuf *pixbuf;
-	                pixbuf = get_button_image (path_bar, button_data->type);
-       	        	gtk_image_set_from_pixbuf (GTK_IMAGE (button_data->image), pixbuf);
+			icon_info = get_custom_user_icon_info (button_data);
+			if (icon_info == NULL) {
+				icon_info = get_type_icon_info (button_data);
+			}
+
+			pixbuf = NULL;
+
+			if (icon_info != NULL) {
+				pixbuf = nautilus_icon_info_get_pixbuf_at_size (icon_info, NAUTILUS_PATH_BAR_ICON_SIZE);
+				g_object_unref (icon_info);
+			}
+
+			if (pixbuf != NULL) {
+				gtk_image_set_from_pixbuf (GTK_IMAGE (button_data->image), pixbuf);
+				gtk_widget_show (GTK_WIDGET (button_data->image));
+				g_object_unref (pixbuf);
+			} else {
+				gtk_widget_hide (GTK_WIDGET (button_data->image));
+			}
 		}
         }
 
+}
+
+static void
+nautilus_path_bar_update_button_state (ButtonData *button_data,
+				       gboolean    current_dir)
+{
+	if (button_data->label != NULL) {
+		g_object_set (button_data->label, "use-markup", current_dir, NULL);
+	}
+
+	nautilus_path_bar_update_button_appearance (button_data);
+
         if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_data->button)) != current_dir) {
                 button_data->ignore_changes = TRUE;
                 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button_data->button), current_dir);
@@ -1281,40 +1389,93 @@ button_drag_data_get_cb (GtkWidget          *widget,
 			 GtkSelectionData   *selection_data,
 			 guint               info,
 			 guint               time_,
-			 gpointer            data)
+			 gpointer            user_data)
 {
         ButtonData *button_data;
-        char *uri_list;
-	char *uri;
+        char *uri_list[2];
+	char *tmp;
 
-        button_data = data;
-	uri = g_file_get_uri (button_data->path);
-        uri_list = g_strconcat (uri, "\r\n", NULL);
-	g_free (uri);
-        gtk_selection_data_set (selection_data,
-			  	selection_data->target,
-			  	8,
-			  	uri_list,
-			  	strlen (uri_list));
-        g_free (uri_list);
+	button_data = user_data;
+
+	uri_list[0] = g_file_get_uri (button_data->path);
+	uri_list[1] = NULL;
+
+	if (info == NAUTILUS_ICON_DND_GNOME_ICON_LIST) {
+		tmp = g_strdup_printf ("%s\r\n", uri_list[0]);
+		gtk_selection_data_set (selection_data, selection_data->target,
+					8, tmp, strlen (tmp));
+		g_free (tmp);
+	} else if (info == NAUTILUS_ICON_DND_URI_LIST) {
+		gtk_selection_data_set_uris (selection_data, uri_list);
+	}
+
+	g_free (uri_list[0]);
+}
+
+static void
+setup_button_drag_source (ButtonData *button_data)
+{
+	GtkTargetList *target_list;
+	const GtkTargetEntry targets[] = {
+		{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST }
+	};
+
+        gtk_drag_source_set (button_data->button,
+		       	     GDK_BUTTON1_MASK,
+		       	     NULL, 0,
+			     GDK_ACTION_MOVE |
+			     GDK_ACTION_COPY |
+			     GDK_ACTION_LINK |
+			     GDK_ACTION_ASK);
+
+	target_list = gtk_target_list_new (targets, G_N_ELEMENTS (targets));
+	gtk_target_list_add_uri_targets (target_list, NAUTILUS_ICON_DND_URI_LIST);
+	gtk_drag_source_set_target_list (button_data->button, target_list);
+	gtk_target_list_unref (target_list);
+
+        g_signal_connect (button_data->button, "drag-data-get",
+			  G_CALLBACK (button_drag_data_get_cb),
+			  button_data);
+}
+
+static void
+button_data_file_changed (NautilusFile *file,
+			  ButtonData *button_data)
+{
+	GFile *location;
+	char *display_name;
+
+	location = nautilus_file_get_location (file);
+	if (!g_file_equal (button_data->path, location)) {
+		button_data->path = g_object_ref (location);
+	}
+	g_object_unref (location);
+
+	display_name = nautilus_file_get_display_name (file);
+	if (eel_strcmp (display_name, button_data->dir_name) != 0) {
+		g_free (button_data->dir_name);
+		button_data->dir_name = g_strdup (display_name);
+	}
+
+	nautilus_path_bar_update_button_appearance (button_data);
+
+	g_free (display_name);
 }
 
 static ButtonData *
 make_directory_button (NautilusPathBar  *path_bar,
-		       const char       *dir_name,
-		       GFile            *path,
+		       NautilusFile     *file,
 		       gboolean          current_dir,
 		       gboolean          base_dir,	
 		       gboolean          file_is_hidden)
 {
-        const GtkTargetEntry targets[] = {
-                { "text/uri-list", 0, 0 }
-        };
-
+	GFile *path;
         GtkWidget *child;
         GtkWidget *label_alignment;
         ButtonData *button_data;
 
+	path = nautilus_file_get_location (file);
+
 	child = NULL;
 	label_alignment = NULL;
 
@@ -1325,17 +1486,20 @@ make_directory_button (NautilusPathBar  *path_bar,
         button_data->type = find_button_type (path_bar, path, button_data);
         button_data->button = gtk_toggle_button_new ();
 	gtk_button_set_focus_on_click (GTK_BUTTON (button_data->button), FALSE);
-	
+	/* TODO update button type when xdg directories change */
+
+	button_data->drag_info.target_location = g_object_ref (path);
+
+	button_data->image = gtk_image_new ();
+
         switch (button_data->type) {
                 case ROOT_BUTTON:
-                        button_data->image = gtk_image_new ();
                         child = button_data->image;
                         button_data->label = NULL;
                         break;
                 case HOME_BUTTON:
                 case DESKTOP_BUTTON:
 		case MOUNT_BUTTON:
-                        button_data->image = gtk_image_new ();
                         button_data->label = gtk_label_new (NULL);
                         label_alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
                         gtk_container_add (GTK_CONTAINER (label_alignment), button_data->label);
@@ -1345,24 +1509,13 @@ make_directory_button (NautilusPathBar  *path_bar,
                         break;
 		case NORMAL_BUTTON:
     		default:
-			if (base_dir) {
-	                        button_data->image = gtk_image_new ();
-        	                button_data->label = gtk_label_new (NULL);
-        	                label_alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-        	                gtk_container_add (GTK_CONTAINER (label_alignment), button_data->label);
-        	                child = gtk_hbox_new (FALSE, 2);
-        	                gtk_box_pack_start (GTK_BOX (child), button_data->image, FALSE, FALSE, 0);
-        	                gtk_box_pack_start (GTK_BOX (child), label_alignment, FALSE, FALSE, 0);
-				button_data->is_base_dir = TRUE;
-				button_data->custom_icon = get_icon_for_file_path (path, NAUTILUS_ICON_FOLDER);
-			} else {
-				button_data->is_base_dir = FALSE;
-	      			button_data->label = gtk_label_new (NULL);
-      				label_alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-      				gtk_container_add (GTK_CONTAINER (label_alignment), button_data->label);
-      				child = label_alignment;
-      				button_data->image = NULL;
-			}
+			button_data->label = gtk_label_new (NULL);
+			label_alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+			gtk_container_add (GTK_CONTAINER (label_alignment), button_data->label);
+			child = gtk_hbox_new (FALSE, 2);
+			gtk_box_pack_start (GTK_BOX (child), button_data->image, FALSE, FALSE, 0);
+			gtk_box_pack_start (GTK_BOX (child), label_alignment, FALSE, FALSE, 0);
+			button_data->is_base_dir = base_dir;
         }
 
   	/* label_alignment is created because we can't override size-request
@@ -1377,8 +1530,16 @@ make_directory_button (NautilusPathBar  *path_bar,
 	
 	/* do not set these for mounts */
 	if (button_data->type != MOUNT_BUTTON) {
-		button_data->dir_name = g_strdup (dir_name);
         	button_data->path = g_object_ref (path);
+		button_data->file = nautilus_file_ref (file);
+		button_data->dir_name = nautilus_file_get_display_name (file);
+		nautilus_file_monitor_add (button_data->file, button_data,
+					   NAUTILUS_FILE_ATTRIBUTES_FOR_ICON);
+		button_data->file_changed_signal_id = 
+			g_signal_connect (button_data->file, "changed",
+					  G_CALLBACK (button_data_file_changed),
+					  button_data);  
+					   
 	}
 
         button_data->file_is_hidden = file_is_hidden;
@@ -1386,17 +1547,17 @@ make_directory_button (NautilusPathBar  *path_bar,
         gtk_container_add (GTK_CONTAINER (button_data->button), child);
         gtk_widget_show_all (button_data->button);
 
-        nautilus_path_bar_update_button_appearance (path_bar, button_data, current_dir);
+        nautilus_path_bar_update_button_state (button_data, current_dir);
 
         g_signal_connect (button_data->button, "clicked", G_CALLBACK (button_clicked_cb), button_data);
         g_object_weak_ref (G_OBJECT (button_data->button), (GWeakNotify) button_data_free, button_data);
 
-        gtk_drag_source_set (button_data->button,
-		       	     GDK_BUTTON1_MASK,
-		       	     targets,
-		             G_N_ELEMENTS (targets),
-		             GDK_ACTION_COPY);
-        g_signal_connect (button_data->button, "drag-data-get",G_CALLBACK (button_drag_data_get_cb), button_data);
+	setup_button_drag_source (button_data);
+
+	nautilus_drag_slot_proxy_init (button_data->button,
+				       &(button_data->drag_info));
+
+	g_object_unref (path);
 
         return button_data;
 }
@@ -1433,7 +1594,8 @@ nautilus_path_bar_check_parent_path (NautilusPathBar *path_bar,
 	      			ButtonData *button_data;
 
 	      			button_data = list->data;
-	      			if (BUTTON_IS_FAKE_ROOT (button_data)) {
+	      			if (list->prev != NULL &&
+				    BUTTON_IS_FAKE_ROOT (button_data)) {
 		  			path_bar->fake_root = list;
 		  			break;
 				}
@@ -1442,9 +1604,8 @@ nautilus_path_bar_check_parent_path (NautilusPathBar *path_bar,
 
                 for (list = path_bar->button_list; list; list = list->next) {
 
-	  		nautilus_path_bar_update_button_appearance (path_bar,
-					  			    BUTTON_DATA (list->data),
-						                    (list == current_path) ? TRUE : FALSE);
+	  		nautilus_path_bar_update_button_state (BUTTON_DATA (list->data),
+							       (list == current_path) ? TRUE : FALSE);
 		}
 
                 if (!gtk_widget_get_child_visible (BUTTON_DATA (current_path->data)->button)) {
@@ -1456,37 +1617,10 @@ nautilus_path_bar_check_parent_path (NautilusPathBar *path_bar,
         return FALSE;
 }
 
-static char *
-get_display_name_for_folder (GFile *location)
-{
-	GFileInfo *info;
-	char *name;
-
-	/* This does sync i/o, which isn't ideal.
-	 * It should probably use the NautilusFile machinery
-	 */
-	
-	name = NULL;
-	info = g_file_query_info (location,
-				  G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
-				  0, NULL, NULL);
-	if (info) {
-		name = g_strdup (g_file_info_get_display_name (info));
-		g_object_unref (info);
-	}
-
-	if (name == NULL) {
-		name = g_file_get_basename (location);
-	}
-	return name;
-}
-
-
 static gboolean
 nautilus_path_bar_update_path (NautilusPathBar *path_bar, GFile *file_path)
 {
-        GFile *path, *parent_path;
-	char *name;
+	NautilusFile *file, *parent_file;
         gboolean first_directory, last_directory;
         gboolean result;
         GList *new_buttons, *l, *fake_root;
@@ -1495,34 +1629,30 @@ nautilus_path_bar_update_path (NautilusPathBar *path_bar, GFile *file_path)
         g_return_val_if_fail (NAUTILUS_IS_PATH_BAR (path_bar), FALSE);
         g_return_val_if_fail (file_path != NULL, FALSE);
 
-        name = NULL;
 	fake_root = NULL;
         result = TRUE;
-	parent_path = NULL;
 	first_directory = TRUE;
 	last_directory = FALSE;
 	new_buttons = NULL;
 
-        path = g_object_ref (file_path);
+	file = nautilus_file_get (file_path);
 
         gtk_widget_push_composite_child ();
 
-        while (path != NULL) {
-
-                parent_path = g_file_get_parent (path);
-                name = get_display_name_for_folder (path);	
-		last_directory = !parent_path;
-                button_data = make_directory_button (path_bar, name, path, first_directory, last_directory, FALSE);
-                g_object_unref (path);
-		g_free (name);
+        while (file != NULL) {
+		parent_file = nautilus_file_get_parent (file);
+		last_directory = !parent_file;
+		button_data = make_directory_button (path_bar, file, first_directory, last_directory, FALSE);
+		nautilus_file_unref (file);
 
                 new_buttons = g_list_prepend (new_buttons, button_data);
 
-		if (BUTTON_IS_FAKE_ROOT (button_data)) {
+		if (parent_file != NULL &&
+		    BUTTON_IS_FAKE_ROOT (button_data)) {
 			fake_root = new_buttons;
 		}
 		
-                path = parent_path;
+		file = parent_file;
                 first_directory = FALSE;
         }
 
