@@ -1,9 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
-
 /*
- * Nautilus
- *
- * Copyright (C) 2000 Eazel, Inc.
+ * Copyright (C) 2005 Novell, Inc.
  *
  * Nautilus is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,23 +17,15 @@
  * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Authors: Maciej Stachowiak <mjs@eazel.com>
- *          Rebecca Schulman <rebecka@eazel.com>
- */
-
-/* nautilus-search-bar.h - Search bar for Nautilus
+ * Author: Anders Carlsson <andersca@imendio.com>
+ *
  */
 
 #ifndef NAUTILUS_SEARCH_BAR_H
 #define NAUTILUS_SEARCH_BAR_H
 
-#include "nautilus-navigation-bar.h"
-
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtklabel.h>
-
-#include <libnautilus-private/nautilus-global-preferences.h>
+#include <gtk/gtk.h>
+#include <libnautilus-private/nautilus-query.h>
 
 #define NAUTILUS_TYPE_SEARCH_BAR (nautilus_search_bar_get_type ())
 #define NAUTILUS_SEARCH_BAR(obj) \
@@ -46,14 +35,27 @@
 #define NAUTILUS_IS_SEARCH_BAR(obj) \
 	GTK_CHECK_TYPE (obj, NAUTILUS_TYPE_SEARCH_BAR)
 
-typedef struct {
-	NautilusNavigationBar parent;
+typedef struct NautilusSearchBarDetails NautilusSearchBarDetails;
+
+typedef struct NautilusSearchBar {
+	GtkEventBox parent;
+	NautilusSearchBarDetails *details;
 } NautilusSearchBar;
 
 typedef struct {
-	NautilusNavigationBarClass parent_class;
+	GtkEventBoxClass parent_class;
+
+	void (* activate) (NautilusSearchBar *bar);
+	void (* cancel)   (NautilusSearchBar *bar);
 } NautilusSearchBarClass;
 
-GType   nautilus_search_bar_get_type (void);
+GType      nautilus_search_bar_get_type     	(void);
+GtkWidget* nautilus_search_bar_new          	(void);
+
+GtkWidget *    nautilus_search_bar_borrow_entry  (NautilusSearchBar *bar);
+void           nautilus_search_bar_return_entry  (NautilusSearchBar *bar);
+void           nautilus_search_bar_grab_focus    (NautilusSearchBar *bar);
+NautilusQuery *nautilus_search_bar_get_query     (NautilusSearchBar *bar);
+void           nautilus_search_bar_clear         (NautilusSearchBar *bar);
 
 #endif /* NAUTILUS_SEARCH_BAR_H */
